@@ -16,9 +16,9 @@ import rasterio, fiona
 import geopandas as gpd
 
 logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(name)s - %(levelname)s: %(message)s')
-import sys
-sys.path.append("../myfunctions")
-import myfunctions.utils as utils
+# import sys
+# sys.path.append("..")
+import modvis.utils as utils
 
 def gantt_plot(ranges, **kwargs):
     """Plot Gantt plot given start and end date.
